/*
 * Copyright (c) 2000 Mark B. Allan. All rights reserved.
 *
 * "Chromium B.S.U." is free software; you can redistribute 
 * it and/or use it and/or modify it under the terms of the 
 * "Clarified Artistic License"
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "gettext.h"

#include "MenuGL.h"

#include <cstdlib>
#include <cstring>
#include <cmath>

#include "compatibility.h"

#ifdef HAVE_APPLE_OPENGL_FRAMEWORK
#include <OpenGL/gl.h>
#else
#ifdef HAVE_IOS
#include <OpenGLES/ES1/gl.h>
#include "ios_util.h"
#else
#include <GL/gl.h>
#endif
#endif

#include "Config.h"

#include "extern.h"
#include "define.h"
#include "Global.h"
#include "HiScore.h"
#include "Ground.h"
#include "Audio.h"
#include "Image.h"

#include "OpenGL_Internal.h"

#include "textGeometry.h"

static const int NumMssg = 5;
static char mssgHelpText[NumMssg][128] = {
N_("  d o n ' t   l e t  -a n y-   e n e m i e s   g e t   p a s t !!"),
N_("  e v e r y   e n e m y   y o u   m i s s   c o s t s   a   l i f e !!"),
N_("  m i s s   p o w e r - u p s  f o r   b i g   p o i n t s !!"),
N_("  c r a s h   i n t o   e n e m i e s   t o   d e s t r o y   t h e m !!"),
N_("  (Chromium ES is a modified version of Chromium BSU)"),
};

//====================================================================
MenuGL::MenuGL()
{
	game = Global::getInstance();
	
	// DEBUG
	checkFrameBuffer();
	
	curSel = NewGame;

	elecStretch = 10.0;
	elecOffX = 0.0;
	elecOffY = 0.0;
	textAngle = 0.0;
	txtHeight = 0.5;
	titleTilt = -10.0;
	
	butHeight	= 0.5;
	butWidth	= butHeight*4.0;
	butOffset	= 3.05;

	menuText[NewGame]	= _("n e w    g a m e");
	menuText[GameLevel]	= _("l e v e l");
	menuText[SkillLevel]= _("s k i l l");
#ifdef HAVE_IOS
#else
	menuText[Graphics]	= _("g f x    d e t a i l");
	menuText[FullScreen]= _("f u l l s c r e e n");
	menuText[ScreenSize]= _("s c r e e n    s i z e");
	menuText[Sound]		= _("s o u n d    f x    v o l u m e");
	menuText[Music]		= _("m u s i c    v o l u m e");
	menuText[MouseSpeed]= _("m o u s e   s p e e d");
	menuText[Quit]		= _("q u i t");
#endif
	
	loadTextures();
	
	thickText = true;

	mssgAlpha = 0.0;
	sprintf(mssgText, " ");
	mssgIndex = 0;
	mssgCount = 0;
	mssgHelpOverride = false;
	
	startMenu();
}

//----------------------------------------------------------
MenuGL::~MenuGL()
{
	deleteTextures();
}

//----------------------------------------------------------
void MenuGL::loadTextures()
{
	elecTex   = Image::load(dataLoc("png/electric.png"), IMG_NOMIPMAPS, IMG_BLEND3, GL_CLAMP, GL_LINEAR, GL_LINEAR);
	backTex   = Image::load(dataLoc("png/menu_back.png"), IMG_NOMIPMAPS, IMG_SOLID, GL_REPEAT, GL_LINEAR, GL_LINEAR);
//	csrTex    = Image::load(dataLoc("png/cursor.png"));
	csrTex    = Image::load(dataLoc("png/heroAmmoFlash00.png"));
	updwnTex  = Image::load(dataLoc("png/menu_updown.png"));
	
#ifdef HAVE_OPENFEINT
	ofTex     = Image::load(dataLoc("png/OF_Logo_Small.png"));
#endif

#ifdef HAVE_GLESv1
	envTex = Image::load(dataLoc("png/reflect.png"), IMG_BUILDMIPMAPS, IMG_SOLID, GL_CLAMP, GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR);
#else
	//-- Environment map
	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);
	
	envTex = Image::load(dataLoc("png/reflect.png"), IMG_BUILDMIPMAPS, IMG_SOLID, GL_CLAMP, GL_LINEAR_MIPMAP_NEAREST, GL_LINEAR);

	listChrom = glGenLists(1);
	listBSU = glGenLists(1);
	createLists( (thickText = true) );
#endif
}

//----------------------------------------------------------
void MenuGL::deleteTextures()
{
	glDeleteTextures(1, &elecTex);
	glDeleteTextures(1, &backTex);

	glDeleteTextures(1, &csrTex);
	glDeleteTextures(1, &updwnTex);
	elecTex	= 0;
	backTex	= 0;
	csrTex	= 0;
	updwnTex = 0;
	
#ifndef HAVE_GLESv1
	glDeleteTextures(1, &envTex);
	envTex	= 0;

	glDeleteLists(listChrom, 1);
	glDeleteLists(listBSU, 1);
#endif
}

//----------------------------------------------------------
void MenuGL::createLists(bool thick)
{
#ifndef HAVE_GLESv1
	Config* config = Config::instance();

	if(!thick)
	{
		if( config->debug() ) fprintf(stderr, _("ATTENTION: Using 'thin' text to improve framerate...\n"));
	}
	titleTilt	= -10.0;
	
	if( config->debug() ) fprintf(stderr, _("MenuGL::createLists\n"));

	glNewList(listChrom, GL_COMPILE);
	textGeometryChromium(thick);
	glEndList();
	
	glNewList(listBSU, GL_COMPILE);
	textGeometryBSU(thick);
	glEndList();
#endif
}

static const char *skillString(int i)
{
	switch(i)
	{
		case 2: return _("fish in a barrel");
		case 3: return _("whimp");
		case 4: return _("easy");
		case 5: return _("normal");
		case 6: return _("experienced");
		case 7: return _("fun");
		case 8: return _("insane");
		case 9: return _("impossible");
		default:return _("-");
	}
}

//----------------------------------------------------------
void MenuGL::startMenu()
{
//	elecOffX	= elecStretch*0.4;
	elecOffX	= 0.0;
	textAngle	= 90.0;
	textCount	= 500;
	createLists( (thickText = true) );
	Global::cursorPos[0] = 0.0;
	Global::cursorPos[1] = 0.0;
	
}

//----------------------------------------------------------
void MenuGL::drawArbQuad(const GLfloat *colors, const GLfloat *vertices)
{
#ifdef HAVE_GLESv1
	glEnableClientState(  GL_VERTEX_ARRAY );
	glEnableClientState(  GL_COLOR_ARRAY );
	glDisableClientState( GL_NORMAL_ARRAY );
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
	
	glVertexPointer(3, GL_FLOAT, 0, vertices);
	glColorPointer(4, GL_FLOAT, 0, colors);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 5);
#else
	glTexCoord2f(1.0, 0.0); 

	glBegin(GL_TRIANGLE_STRIP);
	for (int i=0; i < 5; i++) {
		glColor4f(colors[i*4], colors[i*4 + 1], colors[i*4 + 2], colors[i*4 + 3]);
		glVertex3f(vertixes[i*3], vertices[i*3 + 1], vertices[i*3 + 2]);
	}
	glEnd();
#endif
	CHECK_GL_ERROR();
}

void MenuGL::drawArbTris(int length, const GLfloat *colors, const GLfloat *vertices, const GLfloat *texCoord)
{
#ifdef HAVE_GLESv1
	glDisableClientState( GL_NORMAL_ARRAY );
	glEnableClientState(  GL_VERTEX_ARRAY );
	glEnableClientState(  GL_COLOR_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );

	glTexCoordPointer(2, GL_FLOAT, 0, texCoord);
	glVertexPointer(3, GL_FLOAT, 0, vertices);
	glColorPointer(4, GL_FLOAT, 0, colors);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, length);
#else
	glBegin(GL_TRIANGLE_STRIP);
	for (int i=0; i < length; i++) {
		glTexCoord2f(texCoord[i*2], texCoord[i*2 + 1]);
		glColor4f(colors[i*4], colors[i*4 + 1], colors[i*4 + 2], colors[i*4 + 3]);
		glVertex3f(vertixes[i*3], vertices[i*3 + 1], vertices[i*3 + 2]);
	}
	glEnd();
#endif

	CHECK_GL_ERROR();
}


void MenuGL::drawGL()
{
	Config	*config = Config::instance();
	Global	*game = Global::getInstance();
	HiScore	*hiScore = HiScore::getInstance();
	int		i;
		
	if(--textCount < 0)
	{
		textCount = 500;
//		textAngle = 360.0;
	}
	if(textAngle > 0.0) 
		textAngle -=  5.0;
	else		
		textAngle  =  0.0;
	
//	glClearColor(0.27451, 0.235294, 0.392157, 1.0);
	//-- Clear buffers
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

	//-- Place camera
	glLoadIdentity();
	glTranslatef(0.0, 0.0, config->zTrans());
	
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	
	glColor4f(1.0, 1.0, 1.0, 1.0);
	
	//-- Draw background
	game->ground->drawGL();
	
	//-- Update audio
	game->audio->update();
	
	float	szx		=   9.0;
	float	szy		=   4.5;
	float	top		=   1.0;
	float	left	=  -8.0;
	float	inc		=  -txtHeight*2.5;
	
	//----- Draw credits texture --------------------------------
	glPushMatrix();
		// NOTE: corners of back tex is white, alpha 1 and
		// we are in modulate blend...
		glBindTexture(GL_TEXTURE_2D, backTex);
		
		//-- darken
		const GLfloat colorsA[] = {	
			0.0, 0.0, 0.0, 0.8,
			0.0, 0.0, 0.0, 0.8,
			0.0, 0.0, 0.0, 0.4,
			0.0, 0.0, 0.0, 0.4,
			0.0, 0.0, 0.0, 0.8};

		float dy = txtHeight*NumSelections + 0.75;
	
#ifdef HAVE_OPENFEINT
		dy += 1.0;
#endif
	
		const GLfloat verticesA[] = {
			 szx,  szy+0.25-3.0, 10.0,
			-szx,  szy+0.25-3.0, 10.0,
			-szx,         -dy, 10.0,
 			 szx,         -dy, 10.0,
			 szx,  szy+0.25-3.0, 10.0,};
		// menu background
		drawArbQuad(colorsA, verticesA);

		const GLfloat colorsB[] = {	
			0.0, 0.0, 0.0, mssgAlpha,
			0.0, 0.0, 0.0, mssgAlpha,
			0.0, 0.0, 0.0, mssgAlpha,
			0.0, 0.0, 0.0, mssgAlpha,
			0.0, 0.0, 0.0, mssgAlpha,};
	
		const GLfloat verticesB[] = {
			 16.0, -10.7, 10.0,
			-16.0, -10.7, 10.0,
			-16.0, -11.9, 10.0,
			 16.0, -11.9, 10.0,
			16.0, -10.7, 10.0,};
		drawArbQuad(colorsB, verticesB);
		
		szx = 12.0;
		szy =  txtHeight*0.5;
		glPushMatrix();
		glTranslatef(left, top+(inc*curSel), 10.0);

		const GLfloat colorsC[] = {	
			0.5, 0.5, 1.0, 1.0,
			0.5, 0.5, 1.0, 1.0,
			0.2, 0.2, 1.0, 0.0,
			0.2, 0.2, 1.0, 0.0,
			0.5, 0.5, 1.0, 1.0,};
	
		const GLfloat verticesC[] = {
			-szx,  szy*3.0, 0.0,
			-szx,     -szy, 0.0,
			 szx,     -szy, 0.0,
			 szx,  szy*3.0, 0.0,
			-szx,  szy*3.0, 0.0,};
		drawArbQuad(colorsC, verticesC);
	
		drawIndicator();
		
		glBlendFunc(GL_SRC_ALPHA, GL_ONE);
		drawElectric();
		glPopMatrix();

		//-------- draw the menu text
		glColor4f(1.0, 1.0, 1.0, 0.9);
		float sc = 0.035;
		for(i = 0; i < NumSelections; i++)
		{
			glPushMatrix();
			glTranslatef(left, top+(inc*i), 10.0);
			glRotatef(textAngle, 1.0, 0.0, 0.0);
			glScalef(sc, sc*0.75, 1.0);
			game->text->Render(menuText[i]);
			glPopMatrix();
		}

#ifdef HAVE_OPENFEINT
		{
			// Draw the OpenFeint button
			glPushMatrix();		
			glBindTexture(GL_TEXTURE_2D, ofTex);	
			glTranslatef(-0.6, top+(inc*(NumSelections)), 10.0);
			drawQuad(7.5, 0.8);
			glPopMatrix();
		}
#endif
	
		{
			float f = (float)-game->frame;
			float r = cos(f*0.02);
			glPushMatrix();
//			glColor4f(1.0+r, r, r, 0.6+0.2*r);
			glColor4f(1.0, 1.0, 1.0, 0.6+0.2*r);
			glTranslatef(-18.75, -8.5, 0.0);
			glScalef(sc, sc*0.75, 1.0);
			game->text->Render(_("high scores"));
			glTranslatef(-100.0, -30.0, 0.0);
			char buf[16];
			int i,len;
			float trans;
			time_t nowTime = time(NULL);
			int l = config->intSkill();
			int recentHiScore = -1;
			time_t mostRecent = 0;
			for(i = 0; i < HI_SCORE_HIST; i++)
			{
				if(	hiScore->getDate(l, i) > nowTime-300 && // highlight score for 5 minutes (300)
					hiScore->getDate(l, i) > mostRecent )
				{
					recentHiScore = i;
					mostRecent = hiScore->getDate(l, i);
				}
			}
			for(i = 0; i < HI_SCORE_HIST; i++)
			{
				f += 30.0;
				r = cos(f*0.02);
				if(i == recentHiScore)
					glColor4f(1.5, 0.5, 0.5, 0.6+0.1*r);
				else
					glColor4f(1.0, 1.0, 1.0, 0.2+0.2*r);
//				glColor4f(0.5+r*0.5, 0.5, 0.25-r*0.25, 0.2+0.2*r);
				sprintf(buf, "%d", (int)hiScore->getScore(config->intSkill(), i) );
				len = strlen(buf);
				trans = 80.0 + game->text->Advance(_("high scores")) - game->text->Advance(buf);
				glTranslatef( trans, 0.0, 0.0 );
				game->text->Render(buf);
				glTranslatef( -trans, -30.0, 0.0) ;
			}
			glPopMatrix();
		}
		
		//---- credits
		if(true)
		{
			int		 n = 0;
			float	alpha;
			static double c = 0.0;
			if(c > 75.0)
			{
				c = 0.0;
			}
			c += 0.2;
			glPushMatrix();
			if(c > 25)	alpha = 0.4*(75.0-c)/50.0;
			else		alpha = 0.4;
			glColor4f(1.0, 1.0, 1.0, alpha);
			sc = 0.03;
			glTranslatef(14.0, -11.5, 0.0);
			glScalef(sc, sc, 1.0);
			glTranslatef(-c*1.5, c, 0.0);
			if(c < 3)	n = (int)c;
			else		n = 3;
			if(n>0) game->text->Render(_("the"), n);
			glTranslatef(c, -38+c, 0.0);
			if(c < 10)	n = (int)(c-3);
			else		n = 7;
			if(n>0) game->text->Render(_("reptile"), n);
			glTranslatef(c, -38+c, 0.0);
			if(c < 16)	n = (int)c-10;
			else		n = 6;
			if(n>0) game->text->Render(_("labour"), n);
			glTranslatef(c, -38+c, 0.0);
			if(c < 23)	n = (int)(c-16);
			else		n = 7;
			if(n>0) game->text->Render(_("project"), n);
			// font height is 23
			glPopMatrix();
		}
		
		//-------- draw help message
		if(mssgAlpha > 0.0)
		{
			if(mssgHelpOverride)
				glColor4f(1.3, mssgAlpha, mssgAlpha, mssgAlpha);
			else
				glColor4f(0.5, 0.5, 0.9, (0.2+mssgAlpha));		
			sc = 0.042;
			glTranslatef(-19.5, -14.0, 0.0);
			glScalef(sc, sc*0.75, 1.0);
			size_t len = mbstowcs(NULL,mssgText,0);
			unsigned int	ti = (unsigned int)(112.0*mssgAlpha);
			if(ti > len) ti = len;
			if(ti) game->text->Render(mssgText, ti);
			mssgAlpha -= 0.004;
			glColor4f(1.0, 1.0, 1.0, 1.0);
		}
		
		
	glPopMatrix();
	

	glBlendFunc(GL_SRC_ALPHA, GL_ONE);
	glPushMatrix();
	glTranslatef(0.0, 4.75, 25.0);
	glColor4f(1.0, 1.0, 1.0, 1.0);
	glDepthMask(GL_FALSE);	//XXX Hack to make Voodoo3 XF4 work
	drawTitleBack();
	glDepthMask(GL_TRUE);	//XXX Hack to make Voodoo3 XF4 work
	drawTitle();
	glPopMatrix();
	
//	//-- draw cursor...
//	{
//		float x = Global::cursorPos[0]*16.60;
//		float y = Global::cursorPos[1]*12.45;
//		float z = 10.0;
//		float sz;
//		glBindTexture(GL_TEXTURE_2D, csrTex);
//		sz = 0.2;
//		glBegin(GL_QUADS);
//		glTexCoord2f(1.0, 1.0); glVertex3f( x+sz, y+sz, z);
//		glTexCoord2f(0.0, 1.0); glVertex3f( x-sz, y+sz, z);
//		glTexCoord2f(0.0, 0.0); glVertex3f( x-sz, y-sz, z);
//		glTexCoord2f(1.0, 0.0); glVertex3f( x+sz, y-sz, z);
//		glEnd();
//	}
	
	if(thickText && game->fps < 30)
	{
		if( config->debug() ) fprintf(stderr, _("ATTENTION: Using 'thin' text to improve framerate...\n"));
		createLists( (thickText = false) );	
	}
	if(!thickText && game->fps > 40)
	{
		if( config->debug() ) fprintf(stderr, _("ATTENTION: Reverting to 'thick' text...\n"));
		createLists( (thickText = true) );	
	}
	
	//---------- Help messages
	if(mssgHelpOverride && mssgAlpha < 0.05)
	{
		mssgHelpOverride = false;
		mssgCount = 0;
	}
	if(!mssgHelpOverride)
	{
		int interval = (mssgCount++)%500;
		if(!interval)
		{
			strcpy(mssgText, gettext(mssgHelpText[mssgIndex%NumMssg]));
			mssgIndex++;
		}
		if(interval < 150)
		{
			mssgAlpha = interval/150.0;
		}
	}
}

//----------------------------------------------------------
void MenuGL::drawIndicator()
{
	Config	*config = Config::instance();
	Global	*game = Global::getInstance();
	char	buf[64];
	float	szx = 10.0;
	float	szy = txtHeight;
	float	level = 0.0;
	float	sc = 0.025;
	int		tmp;
	switch(curSel)
	{
		case GameLevel: 
			level = game->gameLevel/9.0;
			sprintf(buf, "%d", game->gameLevel);	 
			break;
		case SkillLevel: 
			level = config->gameSkillBase();
			tmp = (int)((level+0.05)*10.0);
			sprintf(buf, skillString(tmp));
			break;
#ifndef HAVE_GLESv1
		case Graphics: 
			level = config->gfxLevel()/2.0;
			switch(config->gfxLevel())
			{
				case 0: sprintf(buf, _("low")); break;
				case 1: sprintf(buf, _("med")); break;
				case 2: sprintf(buf, _("high")); break;
			}
			break;
		case ScreenSize: 
			level = (float)config->screenSize()/(float)MAX_SCREEN_SIZE; 
			switch(config->screenSize())
			{
				case 0: sprintf(buf, _("512x384")); break;
				case 1: sprintf(buf, _("640x480")); break;
				case 2: sprintf(buf, _("800x600")); break;
				case 3: sprintf(buf, _("1024x768")); break;
				case 4: sprintf(buf, _("1280x960")); break;
			}
			break;
		case FullScreen: 
			level = (float)config->fullScreen(); 
			if(config->fullScreen()) sprintf(buf, _("true"));
			else sprintf(buf, _("false"));
			break;
		case Sound: 
			level = config->volSound(); 
			sprintf(buf, "%d", (int)((level+0.05)*10.0));	 
			break;
		case Music: 
			level = config->volMusic(); 
			sprintf(buf, "%d", (int)((level+0.05)*10.0));	 
			break;
		case MouseSpeed: 
			level = config->mouseSpeed()*10.0; 
			sprintf(buf, "%d", (int)((level+0.005)*100.0));	 
			break;
#endif
		default: 
			level = -5.0;
			break;
	}
	glPushMatrix();
		glTranslatef(0.0, -txtHeight, 0.0);
		const GLfloat colorsA[] = {
			1.0, 1.0, 1.0, 0.3,
			1.0, 1.0, 1.0, 0.3,
			1.0, 1.0, 1.0, 0.15,
			1.0, 1.0, 1.0, 0.15,
			1.0, 1.0, 1.0, 0.3};
	
		const GLfloat verticesA[] = {
			szx+szy, szy, 0.0,
			   -szx, szy, 0.0,
			   -szx, 0.0, 0.0,
  			    szx, 0.0, 0.0,
			szx+szy, szy, 0.0};

		drawArbQuad(colorsA, verticesA);
	
		//-- draw level indicator and/or text
		glPushMatrix();
		if(level > -1.0)
		{
			const GLfloat colorsB[] = {
				1.0, 0.0, 0.0, 1.0,
				1.0, 0.0, 0.0, 1.0,
				0.0, 0.0, 0.0, 0.0,
				0.0, 0.0, 0.0, 0.0,
				1.0, 0.0, 0.0, 1.0};
			const GLfloat verticesB[] = {
				szy+szx*level, szy, 0.0,
						  0.0, szy, 0.0,
				          0.0, 0.0, 0.0,
				    szx*level, 0.0, 0.0,
				szy+szx*level, szy, 0.0};
			drawArbQuad(colorsB, verticesB);
			
			//-- draw +/- buttons ---
			float	bx = butWidth;
			float	by = butHeight;
			float	bo = butOffset;
			glBindTexture(GL_TEXTURE_2D, updwnTex);
			glColor4f(1.0, 1.0, 1.0, 0.6);

#ifdef HAVE_GLESv1			
			const GLfloat texCoordC[] = {
				1.0, 0.0,
				0.0, 0.0,
				0.0, 1.0,
				1.0, 1.0,
				1.0, 0.0,};
			
			const GLfloat verticesC[] = {
				 bx-bo,  by, 0.0,
				0.0-bo,  by, 0.0,
				0.0-bo, 0.0, 0.0,
				 bx-bo, 0.0, 0.0,
				bx-bo,  by, 0.0,};

			glDisableClientState( GL_NORMAL_ARRAY );
			glDisableClientState( GL_COLOR_ARRAY );
			glEnableClientState(  GL_VERTEX_ARRAY );
			glEnableClientState( GL_TEXTURE_COORD_ARRAY );
			
			glVertexPointer(3, GL_FLOAT, 0, verticesC);
			glTexCoordPointer(2, GL_FLOAT, 0, texCoordC);
			glDrawArrays(GL_TRIANGLE_STRIP, 0, 5);
#else			
			glBegin(GL_QUADS);
				glTexCoord2f(1.0, 0.0);	glVertex3f(	 bx-bo,  by, 0.0);
				glTexCoord2f(0.0, 0.0);	glVertex3f(	0.0-bo,  by, 0.0);
				glTexCoord2f(0.0, 1.0);	glVertex3f(	0.0-bo, 0.0, 0.0);
				glTexCoord2f(1.0, 1.0);	glVertex3f(  bx-bo, 0.0, 0.0);
			glEnd();
#endif			
			glColor4f(1.0, 1.0, 1.0, 0.5);
			glTranslatef(11.0, 0.0, 0.0);
			glScalef(sc, sc, 1.0);
			game->text->Render(buf);
		}
		glPopMatrix();
		
	glPopMatrix();
	
}
	
//----------------------------------------------------------
void MenuGL::drawElectric()
{
	float es = elecStretch;
	elecOffY = FRAND*0.7;
	if( (elecOffX+=0.175) > es)
		elecOffX = -5.0;
	float szx = 30.0;
	float szy = txtHeight;
	glBindTexture(GL_TEXTURE_2D, elecTex);
	glPushMatrix();
	glTranslatef(0.0, txtHeight*0.5, 0.0);
	
#ifdef HAVE_GLESv1
	static const GLfloat colors[] = {
		1.0, 1.0, 1.0, 1.0,
		1.0, 1.0, 1.0, 1.0,
		0.0, 0.0, 0.1, 1.0,
		0.0, 0.0, 0.1, 1.0,
		1.0, 1.0, 1.0, 1.0,};
	
	const GLfloat texCoords[] = {
		elecOffX, elecOffY+0.3,
		elecOffX,     elecOffY,
		elecOffX-es,     elecOffY,
		elecOffX-es, elecOffY+0.3,
		elecOffX, elecOffY+0.3,};
	
	const GLfloat vertices[] = {
		 -8.0,  szy, 0.0,
		 -8.0, -szy, 0.0,
		  szx, -szy, 0.0,
		  szx,  szy, 0.0,
		 -8.0,  szy, 0.0,};

	glDisableClientState( GL_NORMAL_ARRAY );
	glEnableClientState( GL_COLOR_ARRAY );
	glEnableClientState(  GL_VERTEX_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	
	glColorPointer(4, GL_FLOAT, 0, colors);
	glVertexPointer(3, GL_FLOAT, 0, vertices);
	glTexCoordPointer(2, GL_FLOAT, 0, texCoords);
	glDrawArrays(GL_TRIANGLE_STRIP, 0, 5);

	glDisableClientState( GL_COLOR_ARRAY );
#else
	glBegin(GL_QUADS);
		glColor4f(1.0, 1.0, 1.0, 1.0);
		glTexCoord2f(   elecOffX, elecOffY+0.3); glVertex3f( -8.0,  szy, 0.0);
		glTexCoord2f(   elecOffX,     elecOffY); glVertex3f( -8.0, -szy, 0.0);
		glColor4f(0.0, 0.0, 0.1, 1.0);
		glTexCoord2f(elecOffX-es,     elecOffY); glVertex3f(  szx, -szy, 0.0);
		glTexCoord2f(elecOffX-es, elecOffY+0.3); glVertex3f(  szx,  szy, 0.0);
	glEnd();
#endif
	glPopMatrix();
}
	
//----------------------------------------------------------
void MenuGL::drawTitleBack()
{
	float	clr_c[4] = { 1.0, 1.0, 1.0, 0.0 };
	float	clr_w[4] = { 1.0, 1.0, 1.0, 1.0 };
	float	w = 9.0;
	float	a = 2.0;
	float	h = 1.4;
	float	z = 0.5;
		
	float	as = a/(w+a);
	float	at = a/(h+a);
	
	glBindTexture(GL_TEXTURE_2D, backTex);
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
	static float amt = 0.0;
	glTranslatef(amt*0.1, amt*0.5, 0.0);
	glRotatef(-amt*100.0, 0.0, 1.0, 1.0);
	amt -= 0.01;
	//-- Top right
	const GLfloat colorsA[] = {
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w),
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c)};
	const GLfloat verticesA[] = {
		w+a,  h  , 0.0,			w+a,  h+a, 0.0,			w  ,  h  , 0.0,
		w  ,  h+a, 0.0,			0.0,  h  , 0.0,			0.0,  h+a, 0.0};
	const GLfloat texCoordA[] = {
		1.0   , 1.0-at,			1.0, 1.0,				1.0-as, 1.0-at,
		1.0-as, 1.0   ,			0.0, 1.0-at,			0.0   , 1.0 };
	drawArbTris(6, colorsA, verticesA, texCoordA);
	

	const GLfloat colorsB[] = {
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w),
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w)};
	const GLfloat texCoordB[] = {
		1.0   , 0.0   ,			1.0   , 1.0-at,			1.0-as, 0.0   ,
		1.0-as, 1.0-at,			0.0   , 0.0 ,			0.0   , 1.0-at};
	const GLfloat verticesB[] = {
		w+a, 0.0, z,			w+a,   h, 0.0,			w  , 0.0, z,
		w  ,	h, 0.0,			0.0, 0.0, z,			0.0,	h, 0.0};
	drawArbTris(6, colorsB, verticesB, texCoordB);
	
	//-- Top left
	const GLfloat colorsC[] = {
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w),
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c)};
	const GLfloat texCoordC[] = {
		0.0   , 1.0-at,			0.0   , 1.0,			1.0-as, 1.0-at,
		1.0-as, 1.0,			1.0   , 1.0-at,			1.0   , 1.0   };
	const GLfloat verticesC[] = {
		0.0, h  , 0.0,			0.0, h+a, 0.0,			-w  , h  , 0.0,
		-w , h+a, 0.0,			-w-a, h, 0.0,			-w-a, h+a, 0.0};
	drawArbTris(6, colorsC, verticesC, texCoordC);
	
	const GLfloat colorsD[] = {
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c)};
	const GLfloat texCoordD[] = {
		0.0   , 0.0,			0.0, 1.0-at,			1.0-as, 0.0,
		1.0-as, 1.0-at,			1.0, 0.0   ,			1.0   , 1.0-at};
	const GLfloat verticesD[] = {
		  0.0, 0.0, z,			0.0,	h, 0.0,			-w  , 0.0, z,
		-w  ,	h, 0.0,			-w-a, 0.0, z,			-w-a,	h, 0.0};
	drawArbTris(6, colorsD, verticesD, texCoordD);

	//-- bottom right
	const GLfloat colorsE[] = {
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w)};
	const GLfloat texCoordE[] = {
		1.0   , 1.0-at,			1.0   , 0.0,			1.0-as, 1.0-at,
		1.0-as, 0.0   ,			0.0   , 1.0-at,			0.0   , 0.0};
	const GLfloat verticesE[] = {
		w+a,  -h, 0.0,			w+a, 0.0, z,			w  ,  -h, 0.0,
		w  , 0.0, z,			0.0,  -h, 0.0,			0.0, 0.0, z  };
	drawArbTris(6, colorsE, verticesE, texCoordE);
	
	const GLfloat colorsF[] = {
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c),
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w)};
	const GLfloat texCoordF[] = {
		1.0   , 1.0,			1.0   , 1.0-at,			1.0-as, 1.0,
		1.0-as, 1.0-at,			0.0   , 1.0   ,			0.0   , 1.0-at};
	const GLfloat verticesF[] = {
		w+a, -h-a, 0.0,			w+a, -h  , 0.0,			w  , -h-a, 0.0,
		w  , -h  , 0.0,			0.0, -h-a, 0.0,			0.0, -h  , 0.0};
	drawArbTris(6, colorsF, verticesF, texCoordF);

	//bottom left
	const GLfloat colorsG[] = {
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_w),
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c)};
	const GLfloat texCoordG[] = {
		0.0   , 1.0-at,			0.0   , 0.0 ,			1.0-as, 1.0-at,
		1.0-as, 0.0   ,			1.0   , 1.0-at,			1.0   , 0.0   };
	const GLfloat verticesG[] = {
		0.0,  -h, 0.0,			0.0, 0.0, z,			-w  ,  -h, 0.0,
		-w  , 0.0, z,			-w-a,  -h, 0.0,			-w-a, 0.0, z};
	drawArbTris(6, colorsG, verticesG, texCoordG);
	
	const GLfloat colorsH[] = {
		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c),
		EXPAND_VEC4(clr_w),		EXPAND_VEC4(clr_c),		EXPAND_VEC4(clr_c)};
	const GLfloat texCoordH[] = {
		0.0   , 1.0,			0.0   , 1.0-at,			1.0-as, 1.0,
		1.0-as, 1.0-at,			1.0   , 1.0   ,			1.0   , 1.0-at};
	const GLfloat verticesH[] = {
		0.0, -h-a, 0.0,			0.0, -h  , 0.0,			-w  , -h-a, 0.0,
		-w  , -h  , 0.0,		-w-a, -h-a, 0.0,		-w-a, -h  , 0.0};
	drawArbTris(6, colorsH, verticesH, texCoordH);
	
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);	
	
}

//----------------------------------------------------------
void MenuGL::drawTitle()
{
	static int tiltCount = 600;
	static float ta0  = -60.0;
	static float ta1  = -90.0;
	float &tilt = titleTilt;
	if(ta0 < 90.0)	ta0 += 0.7;
	else if(!thickText) ta0 += 180.0;
	else ta0 += 5.0;
	if(ta0 > 270.0)	ta0 = ta0-360.0;
		
	if(ta1 < 90.0)	ta1 += 0.55;
	else if(!thickText) ta1 += 180.0;
	else ta1 += 8.0;
	if(ta1 > 270.0)	ta1 = ta1-360.0;
	
	if(thickText)
	{
		tiltCount--;
		if(tiltCount == 0)
			tilt = 360.0+tilt;
		else if(tiltCount < 0)
		{
			tilt -= 1.0;
			if(tilt < -10.0)
			{
				tiltCount = 1500;
				tilt = -10.0;
			}
		}
		else
			tilt -= 0.01;
	}

	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	glColor4f(1.0, 1.0, 1.0, 1.0);
	glPushMatrix();
#ifdef HAVE_GLESv1
	// XXX: TODO: Render title
	glBindTexture(GL_TEXTURE_2D, envTex);
	glPushMatrix();
	glTranslatef(0.0,  1.0, 0.0);
	glRotatef( tilt, 1.0, 0.0, 0.0);
	glRotatef(   ta0, 0.0, 1.0, 0.0);
		textGeometryChromium(thickText);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(0.0, -1.0, 0.0);
	glRotatef( tilt, 1.0, 0.0, 0.0);
	glRotatef(   ta1, 0.0, 1.0, 0.0);
		//textGeometryBSU(thickText);
	glPopMatrix();
	
#else
	glEnable(GL_TEXTURE_GEN_S);
		glEnable(GL_TEXTURE_GEN_T);
		glBindTexture(GL_TEXTURE_2D, envTex);
		glPushMatrix();
			glTranslatef(0.0,  1.0, 0.0);
			glRotatef( tilt, 1.0, 0.0, 0.0);
			glRotatef(   ta0, 0.0, 1.0, 0.0);
			glCallList(listChrom);
		glPopMatrix();
		glPushMatrix();
			glTranslatef(0.0, -1.0, 0.0);
			glRotatef( tilt, 1.0, 0.0, 0.0);
			glRotatef(   ta1, 0.0, 1.0, 0.0);
			glCallList(listBSU);
		glPopMatrix();
		glDisable(GL_TEXTURE_GEN_S);
		glDisable(GL_TEXTURE_GEN_T);
#endif
	glPopMatrix();
	glDisable(GL_DEPTH_TEST);
}

//----------------------------------------------------------
void MenuGL::keyHit(MainToolkit::Key key)
{
	switch(key)
	{
		case MainToolkit::KeyUp:
			if(curSel == (MenuSelection)0)
				curSel = (MenuSelection)(NumSelections-1);
			else
				curSel = (MenuSelection)(curSel - 1);
//			curSel = (MenuSelection)(curSel - 1);
//			if(curSel < (MenuSelection)0)
//				curSel = (MenuSelection)(NumSelections-1);
			elecOffX = 0.0;
			break;
		case MainToolkit::KeyDown:
			curSel = (MenuSelection)((curSel+1)%NumSelections);
			elecOffX = 0.0;
			break;
		case MainToolkit::KeyLeft:
			decItem();
			break;
		case MainToolkit::KeyRight:
			incItem();
			break;
		case MainToolkit::KeyEnter:
			activateItem();
			break;
		default:
			break;
	}
}

//----------------------------------------------------------
void MenuGL::activateItem()
{
	Config	*config = Config::instance();
	Global *game = Global::getInstance();
	switch(curSel)
	{
		case NewGame:
			game->gameMode = Global::Game;
			game->newGame();
			game->toolkit->grabMouse(true);
			game->audio->setMusicMode(Audio::MusicGame);
			break;
		case SkillLevel:
			incItem();
			break;
		case GameLevel:
			incItem();
			break;			
#ifndef HAVE_GLESv1
		case SkillLevel:
			break;
		case GameLevel:
			break;
		case Graphics:
			break;
		case ScreenSize:
			break;
		case FullScreen:
			config->setFullScreen(!config->fullScreen());
			game->deleteTextures();
			if( !game->toolkit->setVideoMode() )
			{
				mssgHelpOverride = true;
				mssgAlpha = 1.1;
				if( config->fullScreen() )
					sprintf(mssgText, _("---- error setting full screen mode ----"));
				else
					sprintf(mssgText, _("---- error setting window mode ----"));
				config->setFullScreen(!config->fullScreen());
			}
			game->loadTextures();
			break;
		case Sound:
			break;
		case Music:
			break;
		case MouseSpeed:
			break;
		case Quit:
			game->game_quit = true;
			break;
		case NumSelections:
			break;
#endif
	}
}

//----------------------------------------------------------
void MenuGL::incItem()
{
	Config	*config = Config::instance();
	HiScore *hiScore = HiScore::getInstance();
	float	pos[3] = { 0.0, 0.0, 25.0 };
	float tmpScoreCalc = 0.0;
	switch(curSel)
	{
		case NewGame:
			activateItem();
			break;
		case SkillLevel:
			tmpScoreCalc = (config->gameSkillBase() + 0.05) * 10.0;
			if (tmpScoreCalc >= 9.4) {
				config->setGameSkillBase(0.2);
			}
			else {
				config->setGameSkillBase(config->gameSkillBase() + 0.1);
			}
			tmpScoreCalc = (config->gameSkillBase() + 0.05) * 10.0;
			
			fprintf(stderr, "skill level: %f:%f", config->gameSkillBase(), tmpScoreCalc);
			
			if( config->debug() ) hiScore->print(config->intSkill());
			game->newGame();
			break;
		case GameLevel:
			game->gameLevel++;
			if(game->gameLevel > config->maxLevel()) 
			{
				mssgHelpOverride = true;
				mssgAlpha = 1.1;
				sprintf(mssgText, _("- you must complete level %d before you can select level %d -"), config->maxLevel(), game->gameLevel);
				game->gameLevel = 1;
			}
			else
				game->newGame();
			break;
#ifndef HAVE_GLESv1
		case Graphics:
			config->setGfxLevel(config->gfxLevel()+1);
			break;
		case ScreenSize:
			config->setScreenSize(config->screenSize()+1);
			game->deleteTextures();
			if( !game->toolkit->setVideoMode() )
			{
				mssgHelpOverride = true;
				mssgAlpha = 1.1;
				sprintf(mssgText, _("---- error setting screen size ----"));
				config->setScreenSize(config->screenSize()-1);
			}
			game->loadTextures();
			break;
		case FullScreen:
			if(!config->fullScreen())
			{
				config->setFullScreen(true);
				game->deleteTextures();
				if( !game->toolkit->setVideoMode() )
				{
					mssgHelpOverride = true;
					mssgAlpha = 1.1;
					sprintf(mssgText, _("---- error setting full screen mode ----"));
					config->setFullScreen(false);
				}
				game->loadTextures();
			}
			break;
		case Sound:
			config->setVolSound(config->volSound()+0.05);
			game->audio->setSoundVolume(config->volSound());
			game->audio->playSound(Audio::Explosion, pos);
			break;
		case Music:
			config->setVolMusic(config->volMusic()+0.05);
			game->audio->setMusicVolume(config->volMusic());
			break;
		case MouseSpeed:
			config->setMouseSpeed(config->mouseSpeed()+0.005);
			break;
		case Quit:
			activateItem();
			break;
		case NumSelections:
			break;
#endif
	}
}

//----------------------------------------------------------
void MenuGL::decItem()
{
#ifdef HAVE_IOS
	incItem();
#else
	Config	*config = Config::instance();
	HiScore *hiScore = HiScore::getInstance();
	float	pos[3] = { 0.0, 0.0, 25.0 };
	switch(curSel)
	{
		case NewGame:
			break;
		case SkillLevel:
			config->setGameSkillBase(config->gameSkillBase()-0.1);
			if( config->debug() ) hiScore->print(config->intSkill());
			game->newGame();
			break;
		case GameLevel:
			game->gameLevel -= 1;
			if(game->gameLevel < 1) 
				game->gameLevel = 1;
			game->newGame();
			break;
#ifndef HAVE_GLESv1
		case Graphics:
			config->setGfxLevel(config->gfxLevel()-1);
			break;
		case ScreenSize:
			config->setScreenSize(config->screenSize()-1);
			game->deleteTextures();
			if( !game->toolkit->setVideoMode() )
			{
				mssgHelpOverride = true;
				mssgAlpha = 1.1;
				sprintf(mssgText, _("---- error setting screen size ----"));
				config->setScreenSize(config->screenSize()+1);
			}
			game->loadTextures();
			break;
		case FullScreen:
			if(config->fullScreen())
			{
				config->setFullScreen(false);
				game->deleteTextures();
				if( !game->toolkit->setVideoMode() )
				{
					mssgHelpOverride = true;
					mssgAlpha = 1.1;
					sprintf(mssgText, _("---- error setting full screen mode ----"));
					config->setFullScreen(true);
				}
				game->loadTextures();
			}
			break;
		case Sound:
			config->setVolSound(config->volSound()-0.05);
			game->audio->setSoundVolume(config->volSound());
			game->audio->playSound(Audio::Explosion, pos);
			break;
		case Music:
			config->setVolMusic(config->volMusic()-0.05);
			game->audio->setMusicVolume(config->volMusic());
			break;
		case MouseSpeed:
			config->setMouseSpeed(config->mouseSpeed()-0.005);
			break;
		case Quit:
			break;
		case NumSelections:
			break;
#endif
	}
#endif //-- HAVE_IOS
}

/**
 * horrible, hacky, but quick to implement....
 */
//----------------------------------------------------------
void MenuGL::mousePress(MainToolkit::Button but, int xi, int yi)
{
	if(but == MainToolkit::Left)
	{
		float x,y;
		Config *config = Config::instance();

		x = xi;
		y = yi;
		
		x /=  (float)config->screenWreal();
		y /=  (float)config->screenHreal();

		x -= 0.5;
		y -= 0.5;
		
		x *= 16.60 ;
		y *= -12.45 * 2;
		
		
		
			//x = -2.0*(0.5-(((float)xi)/config->screenW()))* 16.60 * config->screenWScale();
			//y =  2.0*(0.5-(((float)yi)/config->screenH()))* 12.45 * config->screenHScale();

		float	p = -y+(1.0+txtHeight*1.5);
		float	s = txtHeight*2.5;
		int		mSel = -1;

		
		bool clicked = false;
		
		if(p > 0.0)
		{
			// reset electric 
			elecOffX = 0.0;

			p = p/s;
			mSel = (int)floor(p);
			if( mSel >= 0 && mSel < (int)NumSelections)
			{
				clicked = true;
				if(mSel != (int)curSel)
				{
					curSel = (MenuSelection)mSel;
					elecOffX = 0.0;
					mSel = -1;
				}
			}
			
#ifdef HAVE_OPENFEINT
			if( mSel == (int)NumSelections) {
				activate_openfeint();
			}
#endif
			
		}
		float l  = -8.0 - butOffset;
		float hw = butWidth*0.5;
		if(mSel >= 0 && clicked
		   )
		{
			if(x > l && x < l+hw )
				decItem();
			else if (x > l+hw && x < l+butWidth)
				incItem();
			else
				activateItem();
		}
	}
}

